__author__ = 'Hi'
class BaseConfig(object):

    section = None

    def __init__(self, config):
        print "\n"
        self._global_config = config
        print "[base.BaseConfig.__init__.self._global_config]: ", self._global_config
        print "\n"
        self.config = self._global_config.get(self.section, dict())
        print "base.BaseConfig.__init__.self.section: ", self.section
        print "\n"
        print "[base.BaseConfig.__init__.self.config]: ", self.config
        self.pbr_config = config.get('nrb', dict())
        print "\n"
        print "[base.BaseConfig.__init__.self.pbr_config]: ", self.pbr_config

    def run(self):
        print "self: ", self
        self.hook()
        self.save()
        print "\n"

    def hook(self):
        pass

    def save(self):
        print "The program has entered into the base save method. "
        self._global_config[self.section] = self.config
        print "[pbr.base.BaseConfig.self._global_config[self.section]]: ", self._global_config[self.section]


__author__ = 'Hi'

import os
import re
import sys
import traceback
import ConfigParser as configparser

from distutils import log
from distutils.errors import (DistutilsOptionError, DistutilsModuleError,DistutilsFileError)
from setuptools.dist import Distribution

import nrb.hooks

D1_D2_SETUP_ARGS = {
    "name": ("katamata",),
    "version": ("katamata",),
    "packages": ("files",),
    "scripts": ("files",),
    "cmdclass": ("global", "commands"),
}

# setup() arguments that can have multiple values in setup.cfg
MULTI_FIELDS = ("install_requires", "packages", "scripts", "setup_requires", "cmdclass")


def resolve_name(name):
    print "PROGRAM HAS ENTERED INTO resolve_name method."
    print "\n"
    print "NAME: ", name
    parts = name.split('.')
    print "parts: ", parts
    cursor = len(parts) - 1
    print "cursor: ", cursor

    module_name = parts[:cursor]
    print "module_name: ", module_name
    attr_name = parts[-1]

    while cursor > 0:
        try:
            ret = __import__('.'.join(module_name), fromlist=[attr_name])
            break
        except ImportError:
            print "THERE IS AN ERROR."
            cursor = cursor - 1
            ret = ''

    for part in parts[cursor:]:
        try:
            ret = getattr(ret, part)
        except AttributeError:
            raise ImportError(name)
    print "RETURNED VALUE IS: ", ret
    return ret


def cfg_to_args(path='setup.cfg'):

    parser = configparser.SafeConfigParser()
    if not os.path.exists(path): raise DistutilsFileError("file '%s' does not exist" %os.path.abspath(path))

    parser.read(path)
    config = {} # An empty dictionary

    for section in parser.sections():
        config[section] = dict(parser.items(section))

    nrb.hooks.setup_hook(config)
    print "\n"
    print "Now the program is here."
    kwargs = setup_cfg_to_setup_kwargs(config)

    access = config.get("entry_points")
    print "if 'entry_points' in config[section]: ", access

    if access:
        entry_points = get_entry_points(config)
        print "RETURNED VALUE IS: ", entry_points
        if entry_points:
            kwargs['entry_points'] = entry_points

    print "kwargs: ", kwargs
    return kwargs


def setup_cfg_to_setup_kwargs(config):

    kwargs = {}

    for arg in D1_D2_SETUP_ARGS:
        if len(D1_D2_SETUP_ARGS[arg]) == 2:
            section, option = D1_D2_SETUP_ARGS[arg]

        elif len(D1_D2_SETUP_ARGS[arg]) == 1:
            section = D1_D2_SETUP_ARGS[arg][0]
            option = arg

        in_cfg_value = has_get_option(config, section, option)
        if not in_cfg_value:
            continue

        if arg in MULTI_FIELDS:
            in_cfg_value = split_multiline(in_cfg_value)
        print "SETUP_CFG_TO_SETUP_KWARGS(CONFIG): "
        if in_cfg_value:
            if arg == 'cmdclass':
                print "arg: ", arg
                cmdclass = {}
                dist = Distribution()
                for cls_name in in_cfg_value:
                    print "cls_name: ", cls_name
                    cls = resolve_name(cls_name)
                    print "cls: ", cls
                    cmd = cls(dist)
                    print "cmd: ", cmd
                    cmdclass[cmd.get_command_name()] = cls
                    print "cmdclass: ", cmdclass
                in_cfg_value = cmdclass

        kwargs[arg] = in_cfg_value

    return kwargs

def get_entry_points(config):
    return dict((option, split_multiline(value)) for option, value in config['entry_points'].items())


def has_get_option(config, section, option):
    if section in config and option in config[section]:
        return config[section][option]
    elif section in config and option.replace('_', '-') in config[section]:
        return config[section][option.replace('_', '-')]
    else:
        return False


def split_multiline(value):
    value = [element for element in (line.strip() for line in value.split('\n')) if element]
    return value




__author__ = 'Hi'

from nrb.hooks import base


class CommandsConfig(base.BaseConfig):

    section = 'global'

    def __init__(self, config):
        super(CommandsConfig, self).__init__(config)
        self.commands = self.config.get('commands', "")
        print "[pbr.hooks.commands.CommandsConfig]: ", self.commands
        print "\n"

    def save(self):
        self.config['commands'] = self.commands
        print "[pbr.hooks.commands.CommandsConfig.self.config['commands']]: ", self.config['commands']
        super(CommandsConfig, self).save()

    def add_command(self, command):
        print "pbr.hooks.commands.add_command: ENTERED."
        self.commands = "%s\n%s" % (self.commands, command)
        print "pbr.hooks.commands.add_command.self.commands: ", self.commands
        print "pbr.hooks.commands.add_command: CLOSED."
        print "\n"

    def hook(self):
        print "pbr.hooks.commands.hook: ENTERED"
        self.add_command('nrb.packaging.LocalEggInfo')
        self.add_command('nrb.packaging.LocalSDist')
        self.add_command('nrb.packaging.LocalInstallScripts')
        print "pbr.hooks.commands.hook: CLOSED"
        print "\n"



from __future__ import unicode_literals

from distutils.command import install as du_install
from distutils import log
import os
import sys

import pkg_resources
from setuptools.command import easy_install
from setuptools.command import egg_info
from setuptools.command import install
from setuptools.command import install_scripts
from setuptools.command import sdist

class LocalInstall(install.install):
    command_name = 'install'
    print "THE PROGRAM HAS ENTERED INTO LOCALINSTALL CLASS. "
    def run(self):
        return du_install.install.run(self)

class LocalInstallScripts(install_scripts.install_scripts):
    command_name = 'install_scripts'
    print "THE PROGRAM HAS ENTERED INTO LOCALINSTALLSCRIPT CLASS"
    def run(self):
        import distutils.command.install_scripts

        self.run_command("egg_info")
        print "LOCALINSTALLSCRIPTS [self.run_command]: ", self.run_command

        print "LOCALINSTALLSCRIPTS [self.distribution.scripts]: ", self.distribution.scripts
        if self.distribution.scripts:
            distutils.command.install_scripts.install_scripts.run(self)
            print "LOCALINSTALLSCRIPTS [distutils.command.install_scripts.install_scripts.run(self)]: ", distutils.command.install_scripts.install_scripts.run(self)
        else:
            self.outfiles = []
            print "LOCALINSTALLSCRIPTS [self.outfiles]: ", self.outfiles

        print "LOCALINSTALLSCRIPTS [self.no_ep]: ", self.no_ep
        if self.no_ep:
            return

class LocalManifestMaker(egg_info.manifest_maker):
    print "THE PROGRAM HAS ENTERED INTO LOCALMANIFESTMAKER CLASS."
    def add_defaults(self):
        sdist.sdist.add_defaults(self)
        self.filelist.append(self.template)
        self.filelist.append(self.manifest)
        ei_cmd = self.get_finalized_command('egg_info')
        print "LOCALMANIFESTMAKER [ei_cmd]: ", ei_cmd
        self.filelist.include_pattern("*", prefix=ei_cmd.egg_info)


class LocalEggInfo(egg_info.egg_info): # It is needed in the sdist command, not in install.
    command_name = 'egg_info'
    print "PROGRAM HAS ENTERED INTO LOCALEGGINFO CLASS. : "
    def find_sources(self):
        manifest_filename = os.path.join(self.egg_info, "SOURCES.txt")
        print "LOCALEGGINFO[manifest_filename]: ", manifest_filename
        if (not os.path.exists(manifest_filename) or 'sdist' in sys.argv):
            mm = LocalManifestMaker(self.distribution)
            print "LOCALEGGINFO[mm] : ", mm
            mm.manifest = manifest_filename
            print "LOCALEGGINFO[mm.manifest]: ", mm.manifest
            mm.run()
            self.filelist = mm.filelist
            print "LOCALEGGINFO [self.filelist]: ", self.filelist
        else:
            self.filelist = egg_info.FileList()
            print "LOCALEGGINFO [self.filelist]: ", self.filelist
            for entry in open(manifest_filename, 'r').read().split('\n'):
                self.filelist.append(entry)


class LocalSDist(sdist.sdist):
    command_name = 'sdist'
    print "PROGRAM HAS ENTERED INTO LOCALSDIST class."
    def run(self):
        print "This is the LocalSDist Run Method`"
        sdist.sdist.run(self)




__author__ = 'Hi'
import os

import setuptools

def smart_find_packages(package_list):
    packages = []
    for pkg in package_list.strip().split("\n"):
        print "[FIND_PACKAGE] pkg: ", pkg
        pkg_path = pkg.replace('.', os.path.sep)
        print "[FIND_PACKAGE] pkg_path: ", pkg_path
        packages.append(pkg)
        print "[FIND_PACKAGE] packages.append(pkg): ", packages.append(pkg)
        packages.extend(['%s.%s' % (pkg, f) for f in setuptools.find_packages(pkg_path)])
        print "[FIND_PACKAGE] packages.extend(['%s.%s' % (pkg, f) for f in setuptools.find_packages(pkg_path)]): ", packages.extend(['%s.%s' % (pkg, f) for f in setuptools.find_packages(pkg_path)])
    return "\n".join(set(packages))



__author__ = 'Hi'
import os
import sys

from nrb.hooks import base
from nrb import find_package


class FilesConfig(base.BaseConfig):

    section = 'files'

    def __init__(self, config, name):
        super(FilesConfig, self).__init__(config)
        self.name = name
        self.data_files = self.config.get('data_files', '')

    def save(self):
        self.config['data_files'] = self.data_files
        super(FilesConfig, self).save()


    def hook(self):
        packages = self.config.get('packages', self.name).strip()
        expanded = []
        for pkg in packages.split("\n"):
            if os.path.isdir(pkg.strip()):
                expanded.append(find_package.smart_find_packages(pkg.strip()))
        print "pbr.hooks.files: PACKAGES: ", "\n".join(expanded)
        self.config['packages'] = "\n".join(expanded)




__author__ = 'arun'
from distutils import core
from distutils import errors
import logging
import os
import sys
import warnings

from setuptools import dist

from nrb import util


_saved_core_distribution = core.Distribution


def _monkeypatch_distribution():
    core.Distribution = dist._get_unpatched(core.Distribution)
    print "CORE [core.Distribution]: ", core.Distribution


def _restore_distribution_monkeypatch():
    core.Distribution = _saved_core_distribution


if sys.version_info[0] == 3:
    string_type = str
    integer_types = (int,)
else:
    string_type = basestring
    integer_types = (int, long)


def pbr(dist, attr, value):

    print "CORE [dist]: ", dist
    print "CORE [attr]: ", attr
    print "CORE [value]: ", value

    try:
        _monkeypatch_distribution()
        if not value:
            return

        print "CORE [isinstance(value, string_type)]: ", isinstance(value, string_type)
        if isinstance(value, string_type):
            path = os.path.abspath(value)
        else:
            path = os.path.abspath('setup.cfg')
        print "CORE [path]: ", path


        if not os.path.exists(path): raise errors.DistutilsFileError('The setup.cfg file %s does not exist.' % path)

        # Converts the setup.cfg file to setup() arguments
        try:
            attrs = util.cfg_to_args(path)
            print "CORE [attrs]: ", attrs
        except Exception:
            e = sys.exc_info()[1]
            logging.exception('Error parsing')
            raise errors.DistutilsSetupError('Error parsing %s: %s: %s' % (path, e.__class__.__name__, e))

        if attrs:
            for key, val in attrs.items():
                print "CORE [key]: ", key
                print "CORE [val]: ", val

                print "CORE [hasattr(dist.metadata, 'set_' + key)]: ",hasattr(dist.metadata, 'set_' + key)
                print "CORE [hasattr(dist.metadata, key)]: ",hasattr(dist.metadata, key)
                print "CORE [dist]: ",dist
                print "CORE [hasattr(dist, key)]: ",hasattr(dist, key)

                if hasattr(dist.metadata, 'set_' + key):
                    getattr(dist.metadata, 'set_' + key)(val)
                    print "CORE [getattr(dist.metadata, 'set_' + key)(val)]: ",getattr(dist.metadata, 'set_' + key)(val)
                elif hasattr(dist.metadata, key):
                    setattr(dist.metadata, key, val)
                    print "CORE [setattr(dist.metadata, key, val)]: ",setattr(dist.metadata, key, val)
                elif hasattr(dist, key):
                    setattr(dist, key, val)
                    print "CORE [setattr(dist, key, val)]: ",setattr(dist, key, val)
                else:
                    msg = 'Unknown distribution option: %s' % repr(key)
                    warnings.warn(msg)

        core.Distribution.finalize_options(dist)
        print "CORE [core.Distribution.finalize_options(dist)]: ",core.Distribution.finalize_options(dist)

    finally:
        _restore_distribution_monkeypatch()


from nrb.hooks import commands
from nrb.hooks import files

def setup_hook(config):
    print "[setup_hook]: STARTED"
    print "[setup_hook]: ", config
    commands.CommandsConfig(config).run()
    files.FilesConfig(config, 'nrb').run()
    print "[setup_hook]: FINISHED"

